#!/usr/bin/perl

# Program:	tokenize_eng_src.perl
# Written by:	Dave Graff, LDC
# Purpose:	filter tipster-style sgml source data into tokenized "docset" data

# This version is for English data sources, and watches for special properties of
# English newswire, closed-caption and transcript files.

$Usage = 
    "Usage:  $0 fileid\n".
    "  Reads SGML from file ./src_sgm/fileid.src_sgm\n".
    "  Creates tokenized file in ./tkn/fileid.tkn,\n".
    "  Creates boundary table file in ./tkn_bnd/fileid.tkn_bnd\n";

die $Usage 
    if ( @ARGV != 1 || ! -d "tkn" || ! -d "tkn_bnd" );

$file = shift;
if ( $file =~ /((1998\d{4}_(\d{2})(\d{2}))_\d{4}_([A-Z]+)_[A-Z]+)/i ) {
    $fileid = $1;
    $colldate = $2;
    $bgntime = $3 . ":" . $4;
    $source = $5;
    if ( $file =~ /(\.ccap|\.fdch)/ ) {
	$fileid .= $1;
    }
    $file_bgnsec = &time2sec( $bgntime );
} else {
    die "\nUnsuitable fileid: $file\n\n$Usage";
}

open( INP, "<src_sgm/$fileid.src_sgm" ) ||
    die "can't open file src_sgm/$fileid.src_sgm for input\n\n$Usage";
open( TKN, ">tkn/$fileid.tkn" ) ||
    die "can't open file tkn/$fileid.tkn for output\n\n$Usage";
open( TBL, ">tkn_bnd/$fileid.tkn_bnd" ) ||
    die "can't open file tkn_bnd/$fileid.tkn_bnd for output\n\n$Usage";

$srctype = 
    ( $fileid =~ /(APW|NYT)/ ) ?  "NEWSWIRE" :
    ( $fileid =~ /(CNN|ABC|NBC|MNB)/ ) ?  "CAPTION" : "TRANSCRIPT";

print TBL "<BOUNDSET type=$srctype fileid=$fileid collect_date=$colldate " .
    "collect_src=$source src_lang=ENGLISH content_lang=NATIVE>\n";
print TKN "<DOCSET type=$srctype fileid=$fileid collect_date=$colldate " .
    "collect_src=$source src_lang=ENGLISH content_lang=NATIVE>\n";

$intext = $inannot = $findDL = $recid = 0;

while (<INP>)
{
    if ( ! $intext ) {
	if ( /<DOCNO> (\S+) / ) {
	    $docno = $1;
	    $start = $recid + 1;
	    $use = 1;
	} 
	elsif ( /<DOCTYPE> (\S+) / ) {
	    $doctype = $1;	                    
	    if ( $srctype eq "NEWSWIRE" && $doctype eq "MISCELLANEOUS" ) {
		warn "$fileid:$.:miscellaneous text found in newswire SGML\n";
		$use = 0;
	    }
	}
	elsif ( /<DATE_TIME> \S+ (\S+) / ) {
	    $time = $1;
	    $bgnsec = &time2sec( $time ) - $file_bgnsec;
	}
	elsif ( /<TEXT>/ ) {
	    $intext = $use;	# $intext is 0 for MISC. NEWSWIRE docs, 1 otherwise
	    $findDL++ if ( $srctype eq "NEWSWIRE" ); # watch for dateline in 1st paragraph
	}
	elsif ( /<END_TIME> \S+ (\S+) / ) {
	    $time = $1;
	    $endsec = &time2sec( $time ) - $file_bgnsec;
	}
	elsif ( /<\/DOC>/ ) {
	    $entry = sprintf( "<BOUNDARY docno=%s doctype=%s", $docno, $doctype );
	    if ( $fileid =~ /(CNN|ABC|NBC|MNB|VOA|PRI)/ ) {
		$entry .= sprintf( " Bsec=%4.2f Esec=%4.2f", $bgnsec, $endsec );
		warn "Bsec>Esec in $entry\n" if ( $bgnsec > $endsec );
	    }
	    $entry .= " Brecid=$start Erecid=$recid" if ( $start>0 && $start<=$recid );
	    print TBL "$entry>\n";
	}
	next;
    }
    elsif ( $inannot ) {
	$inannot = 0 if ( /<\/ANNOTATION>/ );
	next;
    }
    else {
	next if ( /<TURN>/ || /<P>/ );
	if ( /<ANNOTATION>/ ) {
	    $inannot = 1 if ( ! /<\/ANNOTATION>/ );
	    next;
	}
	if ( /<\/TEXT>/ ) {
	    $intext = 0;
	    next;
	}
	if ( $findDL > 0 ) { # check for dateline string: "  CAPS, Etc _ First sentence..."
	    if ( /^\s+([A-Z]{2,}[^_]*\_ )[A-Z][ a-z]/o ) {
		$dateline = $1;
		$dateline =~ s/(\W)/\\$1/g;
		s/$dateline//;
	    }
	    $findDL = 0;
	}
	if ( $srctype eq "NEWSWIRE" ) {	# remove any sgml-ized typesetting codes 
	    s/ \&[ULQ][LRC]\; //g;
	}
	s/^\s*//;  # remove leading and trailing white-space
	s/\s*$//;
	@tokens = split;	# tokenize by splitting on white-space
	foreach $t ( @tokens ) 
	{
	    $recid++;
	    print TKN "<W recid=$recid> $t\n";
	}
    }
}

print TKN "</DOCSET>\n";
print TBL "</BOUNDSET>\n";

close TKN;
close TBL;


sub time2sec
{
    ( $hr, $mi, $sec ) = split( /:/, $_[0] );
    $sec += $hr * 3600 + $mi * 60;
    return $sec;
}
